import os
from django.http import HttpResponse
import pandas as pd


def generate_excel_from_sql(sql, columns):
    response = HttpResponse(content_type="application/xls")
    response['Content-Disposition'] = 'attachment; filename=Relatório de Guias.xls'
    try:
        df = pd.read_sql(sql, connection)
        df.to_excel("output.xls", header=columns, index=False)
        with open("output.xls", 'rb') as f:
            excel = f.read()
        response.write(excel)
        os.remove("output.xls")
    finally:
        pass
    return response


import hashlib


def generate_hash_sha1(text):
    hash_object = hashlib.sha1(text.encode('utf-8'))
    hex_dig = hash_object.hexdigest()
    return hex_dig


def base64_encode(text):
    header, base_64 = text.split(",")
    return base_64



# -*- coding: utf-8 -*-
from random import randint


def formatar_cpf(cpf):
    return "%s.%s.%s-%s" % (cpf[0:3], cpf[3:6], cpf[6:9], cpf[9:11])


def generate(formatar=False):
    """
     * Essa função gera um número de CPF válido.
     * @param {Boolean} formatar define se o número do CPF deve ser gerado com os pontos e hífen.
     * @return {String} CPF
     *
     * Regra de Formação
     *
     * O número de um CPF tem exatamente 9 algarismos em sua raiz e mais dois dígitos verificadores que são indicados por último.
     * Portanto, um CPF tem 11 algarismos. O número do CPF é escrito na forma abcdefghi-jk ou diretamente como abcdefghijk onde
     * os algarismos não podem ser todos iguais entre si.
     *
     *                  abc.def.ghi-jk
     *
     * O j é chamado 1° dígito verificador do número do CPF.
     *
     * O k é chamado 2° dígito verificador do número do CPF.
     *
     * Primeiro Dígito
     *
     * Para obter j multiplicamos a, b, c, d, e, f, g, h e i pelas constantes correspondentes, e somamos os resultados de cada multiplicação:
     *
     * S = 10a + 9b + 8c + 7d + 6e + 5f + 4g + 3h + 2i
     *
     * O resultado da soma é dividido por 11, e resto da divisão é tratada da seguinte forma:
     *
     * se o resto for igual a 0 ou 1, j será 0 (zero)
     * se o resto for 2, 3, 4, 5, 6, 7, 8, 9 ou 10, j será 11 - resto
     *
     * Para obter k, multiplicamos a, b, c, d, e, f, g, h, i e j pelas constantes correspondentes, e somamos os resultados de cada multiplicação:
     *
     * S = 11a + 10b + 9c + 8d + 7e + 6f + 5g + 4h + 3i + 2j
     *
     * O resultado da soma é dividido por 11, e resto da divisão é tratada da seguinte forma:
     *
     * se o resto for igual a 0 ou 1, k será 0 (zero)
     * se o resto for 2, 3, 4, 5, 6, 7, 8, 9 ou 10, k será 11 - resto
     *
    """
    # 9 números aleatórios
    arNumeros = []
    for i in range(9):
        arNumeros.append(randint(0, 9))
    
    # Calculado o primeiro DV
    somaJ = (arNumeros[0] * 10) + (arNumeros[1] * 9) + (arNumeros[2] * 8) + (arNumeros[3] * 7) + (arNumeros[4] * 6) + (
            arNumeros[5] * 5) + (arNumeros[6] * 4) + (arNumeros[7] * 3) + (arNumeros[8] * 2)
    
    restoJ = somaJ % 11
    
    if restoJ == 0 or restoJ == 1:
        j = 0
    else:
        j = 11 - restoJ
    
    arNumeros.append(j)
    
    # Calculado o segundo DV
    somaK = (arNumeros[0] * 11) + (arNumeros[1] * 10) + (arNumeros[2] * 9) + (arNumeros[3] * 8) + (arNumeros[4] * 7) + (
            arNumeros[5] * 6) + (arNumeros[6] * 5) + (arNumeros[7] * 4) + (arNumeros[8] * 3) + (j * 2)
    
    restoK = somaK % 11
    
    if restoK == 0 or restoK == 1:
        k = 0
    else:
        k = 11 - restoK
    
    arNumeros.append(k)
    
    cpf = ''.join(str(x) for x in arNumeros)
    
    if formatar:
        return cpf[:3] + '.' + cpf[3:6] + '.' + cpf[6:9] + '-' + cpf[9:]
    else:
        return cpf


from marshmallow import fields, missing


class SmartNested(fields.Nested):
    
    def _validate_missing(self, value):
        if value is missing and getattr(self, 'required', False):
            self.fail('required')
        return super()._validate_missing(value)
    
    def serialize(self, attr, obj, accessor=None):
        if obj is not None and not isinstance(obj, list) and attr not in obj.__dict__:
            foreign_key_name = None
            if hasattr(obj, attr + '_id'):
                foreign_key_name = attr + '_id'
            elif hasattr(obj, 'id_' + attr):
                foreign_key_name = 'id_' + attr
            if foreign_key_name:
                value = getattr(obj, foreign_key_name)
                if value:
                    return {'id': int(value)}
        return super(SmartNested, self).serialize(attr, obj, accessor)



from django.utils import timezone
from rest_framework import serializers
from rest_framework import status
from rest_framework.response import Response
from rest_framework.viewsets import ModelViewSet


class BaseModelSerializer(serializers.ModelSerializer):
    def __init__(self, *args, **kwargs):
        remove_fields = kwargs.pop('remove_fields', None)
        super().__init__(*args, **kwargs)
        if remove_fields:
            for field_name in remove_fields:
                self.fields.pop(field_name)
    
                
class BaseDao:
    __queryset = None
    
    def save(self, objeto):
        objeto.save()
    
    @property
    def queryset(self):
        return self.__queryset
    
    @queryset.setter
    def queryset(self, queryset):
        self.__queryset = queryset
    
    def get_all(self):
        return self.queryset.all()
    
    def get_all_by_fields(self, **fields):
        return self.queryset.filter(**fields)


class BaseModelViewSet(ModelViewSet):
    
    def perform_create(self, serializer):
        fields = {}
        if hasattr(serializer.Meta.model, 'user'):
            fields['user'] = self.request.user
        serializer.save(**fields)
    
    def perform_update(self, serializer):
        fields = {}
        if hasattr(serializer.Meta.model, 'user'):
            fields['user'] = self.request.user
        if hasattr(serializer.Meta.model, 'data_alt'):
            fields['data_alt'] = timezone.now()
        serializer.save(**fields)
    
    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        if hasattr(instance, 'ativo'):
            instance.ativo = 0
            if hasattr(instance, 'data_fim'):
                instance.data_fim = timezone.now()
            instance.save()
        else:
            self.perform_destroy(instance)
        
        return Response(status=status.HTTP_204_NO_CONTENT)



from .services import timedelta_to_str, Days, get_previous_sunday, daterange, last_day_of_week, week_of_month, \
    get_next_sunday, week_sum, first_day_of_week, month_subtract, first_day_of_month, convert_timedelta_to_time, \
    count_sundays, last_day_of_month, hourrange, round_time, convert_date, count_days_of_month, date_to_float, \
    random_date_between


import requests
from datetime import datetime

__URL_SERVER = "http://webserver.mateus:8080"


def download(hash):
    url = __URL_SERVER + "/download?hash={0}".format(hash)
    response = requests.get(url)
    return response


def upload(arquivo, categoria="", observacao="", nome_arquivo="", senha=""):
    url = __URL_SERVER + "/upload"
    prefixo = str(datetime.now().date())+'-'+nome_arquivo
    headers = {
        'ws-categoria': categoria,
        'ws-arquivo': prefixo,
        'ws-observacao': observacao,
        'ws-senha': senha,
        'content-disposition': 'form-data; name="data" filename="data"'
    }
    response = requests.post(url=url, files={'data': arquivo}, headers=headers)
    return response



# coding=utf-8
import os
import os.path
import shutil
from subprocess import call
from tempfile import mkdtemp, mkstemp

from django.http import HttpResponse
from django.template.loader import render_to_string


def generate_pdf_from_template(template):
    """ Recebe um template e gera um PDF a partir dele"""
    original_path = os.getcwd()  # é necessário armazenar o caminho anterior para restaurar depois
    r = HttpResponse(content_type='application/pdf')
    try:
        tmp_folder = mkdtemp()
        os.chdir(tmp_folder)
        file, filename = mkstemp(dir=tmp_folder)
        os.write(file, template)
        os.close(file)
        call(['pdflatex', filename])
        with open(os.path.join(tmp_folder, filename + '.pdf'), 'rb') as f:
            pdf = f.read()
        # Remove intermediate files
        os.remove(filename)
        os.remove(filename + '.aux')
        os.remove(filename + '.log')
        r.write(pdf)
        shutil.rmtree(tmp_folder)
    finally:
        os.chdir(original_path)
    return r


def replace_caracter_especial(string):
    import re
    new_string = re.sub(r'([^.a-zA-Z0-9áàâãéèêíïóôõöúçñÁÀÂÃÉÈÍÏÓÔÕÖÚÇÑ@&\\\-/{}():;,?*%$ ])', "", string)
    return new_string.replace("&", "\&").encode("utf-8").decode("utf-8", "replace")



import itertools
import math

from flask import current_app
from flask import url_for
from sqlalchemy import exc as sa_exc, inspect
from sqlalchemy import util, log
from sqlalchemy.orm import properties
from sqlalchemy.orm import (
    util as orm_util
)
from sqlalchemy.orm.strategies import AbstractRelationshipLoader


class Page(object):
    
    def __init__(self, items, page, page_size, total):
        self.items = items
        self.previous_page = None
        self.next_page = None
        self.has_previous = page > 1
        if self.has_previous:
            self.previous_page = page - 1
        previous_items = (page - 1) * page_size
        self.has_next = previous_items + len(items) < total
        if self.has_next:
            self.next_page = page + 1
        self.total = total
        self.pages = int(math.ceil(total / float(page_size)))


def paginate(query, page, page_size):
    if page <= 0:
        raise AttributeError('page needs to be >= 1')
    if page_size <= 0:
        raise AttributeError('page_size needs to be >= 1')
    items = query.limit(page_size).offset((page - 1) * page_size).all()
    total = query.order_by(None).count()
    return Page(items, page, page_size, total)


class Pagination:
    def __init__(self, request, query, blue_print, nome_da_view, key_name, schema):
        self.request = request
        self.query = query
        self.resource_for_url = blue_print + '.' + nome_da_view.__name__
        self.key_name = key_name
        self.schema = schema
        self.results_per_page = current_app.config['PAGINATION_PAGE_SIZE']
        self.page_argument_name = current_app.config['PAGINATION_PAGE_ARGUMENT_NAME']
    
    def paginate_query(self):
        page_number = self.request.args.get(self.page_argument_name, 1, type=int)
        
        paginated_objects = paginate(self.query, page_number, self.results_per_page)
        objects = paginated_objects.items
        if paginated_objects.has_previous:
            previous_page_url = url_for(
                self.resource_for_url,
                page=page_number - 1,
                _external=True)
        else:
            previous_page_url = None
        if paginated_objects.has_next:
            next_page_url = url_for(
                self.resource_for_url,
                page=page_number + 1,
                _external=True)
        else:
            next_page_url = None
        dumped_objects = self.schema.dump(objects, many=True).data
        return ({
            self.key_name: dumped_objects,
            'previous': previous_page_url,
            'next': next_page_url,
            'count': paginated_objects.total
        })


def create_object_from_sql_alchemy_row(row, instance):
    """ Monta objeto a partir de uma linha de query do SQL Alchemy"""
    fields = {field.key: atrr for atrr, field in instance.__mapper__.columns.items()}
    for column in row.keys():
        if column in fields:
            setattr(instance, fields[column], row[column])
    return instance


@log.class_logger
@properties.RelationshipProperty.strategy_for(lazy="subquery_with_nolock")
class SubqueryLoaderWithNoLock(AbstractRelationshipLoader):
    __slots__ = 'join_depth',
    
    def __init__(self, parent, strategy_key):
        super(SubqueryLoaderWithNoLock, self).__init__(parent, strategy_key)
        self.join_depth = self.parent_property.join_depth
    
    def init_class_attribute(self, mapper):
        self.parent_property._get_strategy((("lazy", "select"),)).init_class_attribute(mapper)
    
    def setup_query(
            self, context, entity,
            path, loadopt, adapter,
            column_collection=None,
            parentmapper=None, **kwargs):
        
        if not context.query._enable_eagerloads:
            return
        elif context.query._yield_per:
            context.query._no_yield_per("subquery")
        
        path = path[self.parent_property]
        
        # build up a path indicating the path from the leftmost
        # entity to the thing we're subquery loading.
        with_poly_info = path.get(
            context.attributes,
            "path_with_polymorphic", None)
        if with_poly_info is not None:
            effective_entity = with_poly_info.entity
        else:
            effective_entity = self.mapper
        
        subq_path = context.attributes.get(
            ('subquery_path', None),
            orm_util.PathRegistry.root)
        
        subq_path = subq_path + path
        
        # if not via query option, check for
        # a cycle
        if not path.contains(context.attributes, "loader"):
            if self.join_depth:
                if path.length / 2 > self.join_depth:
                    return
            elif subq_path.contains_mapper(self.mapper):
                return
        
        leftmost_mapper, leftmost_attr, leftmost_relationship = \
            self._get_leftmost(subq_path)
        
        orig_query = context.attributes.get(
            ("orig_query", SubqueryLoaderWithNoLock),
            context.query)
        
        # generate a new Query from the original, then
        # produce a subquery from it.
        left_alias = self._generate_from_original_query(
            orig_query, leftmost_mapper,
            leftmost_attr, leftmost_relationship,
            entity.entity_zero
        )
        
        # generate another Query that will join the
        # left alias to the target relationships.
        # basically doing a longhand
        # "from_self()".  (from_self() itself not quite industrial
        # strength enough for all contingencies...but very close)
        q = orig_query.session.query(effective_entity).with_hint(effective_entity, 'WITH (NOLOCK)')
        q._attributes = {
            ("orig_query", SubqueryLoaderWithNoLock): orig_query,
            ('subquery_path', None): subq_path
        }
        
        q = q._set_enable_single_crit(False)
        to_join, local_attr, parent_alias = \
            self._prep_for_joins(left_alias, subq_path)
        q = q.order_by(*local_attr)
        q = q.add_columns(*local_attr)
        q = self._apply_joins(
            q, to_join, left_alias,
            parent_alias, effective_entity)
        
        q = self._setup_options(q, subq_path, orig_query, effective_entity)
        q = self._setup_outermost_orderby(q)
        
        # add new query to attributes to be picked up
        # by create_row_processor
        path.set(context.attributes, "subquery", q)
    
    def _get_leftmost(self, subq_path):
        subq_path = subq_path.path
        subq_mapper = orm_util._class_to_mapper(subq_path[0])
        
        # determine attributes of the leftmost mapper
        if self.parent.isa(subq_mapper) and \
                self.parent_property is subq_path[1]:
            leftmost_mapper, leftmost_prop = \
                self.parent, self.parent_property
        else:
            leftmost_mapper, leftmost_prop = \
                subq_mapper, \
                subq_path[1]
        
        leftmost_cols = leftmost_prop.local_columns
        
        leftmost_attr = [
            getattr(
                subq_path[0].entity,
                leftmost_mapper._columntoproperty[c].key)
            for c in leftmost_cols
        ]
        
        return leftmost_mapper, leftmost_attr, leftmost_prop
    
    def _generate_from_original_query(
            self,
            orig_query, leftmost_mapper,
            leftmost_attr, leftmost_relationship, orig_entity
    ):
        # reformat the original query
        # to look only for significant columns
        q = orig_query._clone().correlate(None)
        
        # set a real "from" if not present, as this is more
        # accurate than just going off of the column expression
        if not q._from_obj and orig_entity.mapper.isa(leftmost_mapper):
            q._set_select_from([orig_entity], False)
        target_cols = q._adapt_col_list(leftmost_attr)
        
        # select from the identity columns of the outer
        q._set_entities(target_cols)
        
        distinct_target_key = leftmost_relationship.distinct_target_key
        
        if distinct_target_key is True:
            q._distinct = True
        elif distinct_target_key is None:
            # if target_cols refer to a non-primary key or only
            # part of a composite primary key, set the q as distinct
            for t in set(c.table for c in target_cols):
                if not set(target_cols).issuperset(t.primary_key):
                    q._distinct = True
                    break
        
        if q._order_by is False:
            q._order_by = leftmost_mapper.order_by
        
        # don't need ORDER BY if no limit/offset
        if q._limit is None and q._offset is None:
            q._order_by = None
        
        # the original query now becomes a subquery
        # which we'll join onto.
        
        embed_q = q.with_labels().subquery()
        left_alias = orm_util.AliasedClass(
            leftmost_mapper, embed_q,
            use_mapper_path=True)
        return left_alias
    
    def _prep_for_joins(self, left_alias, subq_path):
        # figure out what's being joined.  a.k.a. the fun part
        to_join = []
        pairs = list(subq_path.pairs())
        
        for i, (mapper, prop) in enumerate(pairs):
            if i > 0:
                # look at the previous mapper in the chain -
                # if it is as or more specific than this prop's
                # mapper, use that instead.
                # note we have an assumption here that
                # the non-first element is always going to be a mapper,
                # not an AliasedClass
                
                prev_mapper = pairs[i - 1][1].mapper
                to_append = prev_mapper if prev_mapper.isa(mapper) else mapper
            else:
                to_append = mapper
            
            to_join.append((to_append, prop.key))
        
        # determine the immediate parent class we are joining from,
        # which needs to be aliased.
        
        if len(to_join) < 2:
            # in the case of a one level eager load, this is the
            # leftmost "left_alias".
            parent_alias = left_alias
        else:
            info = inspect(to_join[-1][0])
            if info.is_aliased_class:
                parent_alias = info.entity
            else:
                # alias a plain mapper as we may be
                # joining multiple times
                parent_alias = orm_util.AliasedClass(
                    info.entity,
                    use_mapper_path=True)
        
        local_cols = self.parent_property.local_columns
        
        local_attr = [
            getattr(parent_alias, self.parent._columntoproperty[c].key)
            for c in local_cols
        ]
        return to_join, local_attr, parent_alias
    
    def _apply_joins(
            self, q, to_join, left_alias, parent_alias,
            effective_entity):
        
        ltj = len(to_join)
        if ltj == 1:
            to_join = [
                getattr(left_alias, to_join[0][1]).of_type(effective_entity)
            ]
        elif ltj == 2:
            to_join = [
                getattr(left_alias, to_join[0][1]).of_type(parent_alias),
                getattr(parent_alias, to_join[-1][1]).of_type(effective_entity)
            ]
        elif ltj > 2:
            middle = [
                (
                    orm_util.AliasedClass(item[0])
                    if not inspect(item[0]).is_aliased_class
                    else item[0].entity,
                    item[1]
                ) for item in to_join[1:-1]
            ]
            inner = []
            
            while middle:
                item = middle.pop(0)
                attr = getattr(item[0], item[1])
                if middle:
                    attr = attr.of_type(middle[0][0])
                else:
                    attr = attr.of_type(parent_alias)
                
                inner.append(attr)
            
            to_join = [
                          getattr(left_alias, to_join[0][1]).of_type(inner[0].parent)
                      ] + inner + [
                          getattr(parent_alias, to_join[-1][1]).of_type(effective_entity)
                      ]
        
        for attr in to_join:
            q = q.join(attr, from_joinpoint=True)
        return q
    
    def _setup_options(self, q, subq_path, orig_query, effective_entity):
        # propagate loader options etc. to the new query.
        # these will fire relative to subq_path.
        q = q._with_current_path(subq_path)
        q = q._conditional_options(*orig_query._with_options)
        if orig_query._populate_existing:
            q._populate_existing = orig_query._populate_existing
        
        return q
    
    def _setup_outermost_orderby(self, q):
        if self.parent_property.order_by:
            # if there's an ORDER BY, alias it the same
            # way joinedloader does, but we have to pull out
            # the "eagerjoin" from the query.
            # this really only picks up the "secondary" table
            # right now.
            eagerjoin = q._from_obj[0]
            eager_order_by = \
                eagerjoin._target_adapter. \
                    copy_and_process(
                    util.to_list(
                        self.parent_property.order_by
                    )
                )
            q = q.order_by(*eager_order_by)
        return q
    
    class _SubqCollections(object):
        """Given a :class:`.Query` used to emit the "subquery load",
        provide a load interface that executes the query at the
        first moment a value is needed.

        """
        _data = None
        
        def __init__(self, subq):
            self.subq = subq
        
        def get(self, key, default):
            if self._data is None:
                self._load()
            return self._data.get(key, default)
        
        def _load(self):
            self._data = dict(
                (k, [vv[0] for vv in v])
                for k, v in itertools.groupby(
                    self.subq,
                    lambda x: x[1:]
                )
            )
        
        def loader(self, state, dict_, row):
            if self._data is None:
                self._load()
    
    def create_row_processor(
            self, context, path, loadopt,
            mapper, result, adapter, populators):
        if not self.parent.class_manager[self.key].impl.supports_population:
            raise sa_exc.InvalidRequestError(
                "'%s' does not support object "
                "population - eager loading cannot be applied." %
                self)
        
        path = path[self.parent_property]
        
        subq = path.get(context.attributes, 'subquery')
        
        if subq is None:
            return
        
        assert subq.session is context.session, (
            "Subquery session doesn't refer to that of "
            "our context.  Are there broken context caching "
            "schemes being used?"
        )
        
        local_cols = self.parent_property.local_columns
        
        # cache the loaded collections in the context
        # so that inheriting mappers don't re-load when they
        # call upon create_row_processor again
        collections = path.get(context.attributes, "collections")
        if collections is None:
            collections = self._SubqCollections(subq)
            path.set(context.attributes, 'collections', collections)
        
        if adapter:
            local_cols = [adapter.columns[c] for c in local_cols]
        
        if self.uselist:
            self._create_collection_loader(
                context, collections, local_cols, populators)
        else:
            self._create_scalar_loader(
                context, collections, local_cols, populators)
    
    def _create_collection_loader(
            self, context, collections, local_cols, populators):
        def load_collection_from_subq(state, dict_, row):
            collection = collections.get(
                tuple([row[col] for col in local_cols]),
                ()
            )
            state.get_impl(self.key). \
                set_committed_value(state, dict_, collection)
        
        populators["new"].append((self.key, load_collection_from_subq))
        if context.invoke_all_eagers:
            populators["eager"].append((self.key, collections.loader))
    
    def _create_scalar_loader(
            self, context, collections, local_cols, populators):
        def load_scalar_from_subq(state, dict_, row):
            collection = collections.get(
                tuple([row[col] for col in local_cols]),
                (None,)
            )
            if len(collection) > 1:
                util.warn(
                    "Multiple rows returned with "
                    "uselist=False for eagerly-loaded attribute '%s' "
                    % self)
            
            scalar = collection[0]
            state.get_impl(self.key). \
                set_committed_value(state, dict_, scalar)
        
        populators["new"].append((self.key, load_scalar_from_subq))
        if context.invoke_all_eagers:
            populators["eager"].append((self.key, collections.loader))


# -*- coding: utf-8 -*-
from flask import jsonify
from werkzeug.exceptions import HTTPException


def handle_generic_error(e):
    codigo = 500
    descricao = "Erro interno no servidor"
    if isinstance(e, HTTPException):
        codigo = e.code
        descricao = e.description
    
    erro = {
        'description': descricao,
        'error': type(e).__name__,
        'status_code': codigo
    }
    return jsonify(erro), codigo


def responder_com_erro(e=None, mensagem='', status=400):
    codigo = status
    if e and isinstance(e, HTTPException):
        codigo = e.code
        mensagem = e.description
    
    erro = {
        'mensagem': mensagem,
        'erro': type(e).__name__ if not type(e).__name__ else 'ocorreu um erro',
        'status_code': codigo
    }
    return jsonify(erro), codigo


class InvalidUsage(Exception):
    status_code = 400
    
    def __init__(self, description, error, status_code=None, payload=None):
        Exception.__init__(self)
        self.description = description
        self.error = error
        if status_code is not None:
            self.status_code = status_code
        self.payload = payload
    
    def to_dict(self):
        rv = dict(self.payload or ())
        rv['description'] = self.description
        rv['error'] = self.error
        rv['status_code'] = self.status_code
        return rv



from datetime import datetime, timedelta
from dateutil import parser
import sys
import traceback
from enum import IntEnum
from random import randrange
from dateutil.relativedelta import relativedelta, SU, SA
from math import ceil


class Days(IntEnum):
    MONDAY = 0
    TUESDAY = 1
    WEDNESDAY = 2
    THURSDAY = 3
    FRIDAY = 4
    SATURDAY = 5
    SUNDAY = 6


def convert_date(str_date):
    try:
        if str_date:
            
            if not isinstance(str_date, str):
                return str_date
            
            date = parser.parse(str_date)
            if date.year + 100 > datetime.today().year:
                return date
        return datetime.today()
    except:
        traceback.print_exc(file=sys.stdout)
        return datetime.today()


def last_day_of_month(month):
    next_month = month.replace(day=28) + timedelta(days=4)
    return next_month - timedelta(days=next_month.day)


def first_day_of_month(month):
    return month.replace(day=1)


def get_previous_sunday(day):
    return day + relativedelta(weekday=SU(-1 if day.weekday() != Days.SUNDAY else -2))


# diff = (day.weekday() + 1) % 7
# return day - timedelta(7 + diff)


def get_next_sunday(day):
    return day + relativedelta(weekday=SA(1 if day.weekday() != Days.SATURDAY else 2))


# diff = (day.weekday() + 1) % 7
# return day - timedelta(7 + diff)


def random_date_between(begin_date, end_date):
    """
    This function will return a random datetime between two datetime
    objects.
    """
    delta = end_date - begin_date
    int_delta = (delta.days * 24 * 60 * 60) + delta.seconds
    if int_delta > 0:
        random_second = randrange(int_delta)
        return begin_date + timedelta(seconds=random_second)
    return begin_date


def round_time(dt, round_to=60):
    """Round a datetime object to any time laps in seconds
    dt : datetime.datetime object
    round_to : Closest number of seconds to round to, default 1 minute.
    """
    seconds = (dt.replace(tzinfo=None) - dt.min).seconds
    rounding = (seconds + round_to / 2) // round_to * round_to
    return dt + timedelta(0, rounding - seconds, -dt.microsecond)


def daterange(start_date, end_date):
    """Itera datas em um periodo especifico"""
    for n in range(int((end_date - start_date).days + 1)):
        yield start_date + timedelta(n)


def hourrange(start_hour, end_hour):
    """Itera Horas em um periodo especifico"""
    for n in range(0, int((end_hour - start_hour).seconds + 3600), 3600):
        yield start_hour + timedelta(seconds=n)


def count_sundays(begin_date, end_date):
    """Retorna a quantidade de domingos dentro de um periodo especifico"""
    count = 0
    for date in daterange(begin_date, end_date):
        if date.weekday() == int(Days.SUNDAY):
            count += 1
    return count


def month_subtract(date, count_months):
    """ Subtrai meses de uma data"""
    day = date.day
    new_date = date
    for i in range(count_months):
        new_date = new_date.replace(day=1) - timedelta(days=1)
    return new_date.replace(day=day)


def week_sum(date, count_weeks):
    """ Soma semanas em uma determinada data, considerando que uma semana possui 7 dias"""
    new_date = date
    for i in range(count_weeks):
        new_date += timedelta(7)
    return new_date


def date_to_float(date):
    if type(date) is timedelta:
        return date.seconds // 3600 + (date.seconds // 60) % 60 / 100
    return date.hour + date.minute / 100


def first_day_of_week(date):
    first_day = date
    while first_day.weekday() != Days.SUNDAY:
        first_day -= timedelta(1)
    return first_day


def last_day_of_week(date):
    last_day = date
    while last_day.weekday() != Days.SATURDAY:
        last_day += timedelta(1)
    return last_day


def count_days_of_month(month):
    count = 0
    for day in daterange(first_day_of_month(month), last_day_of_month(month)):
        count += 1
    return count


def convert_timedelta_to_time(date_timedelta):
    return (datetime.min + date_timedelta).time()


def week_of_month(date):
    """ Returns the week of the month for the specified date. """
    first_day = date.replace(day=1)
    dom = date.day
    adjusted_dom = dom + first_day.weekday()
    return int(ceil(adjusted_dom / 7.0))


def timedelta_to_str(dt_timedelta):
    """ Converte timedelta para string, colocando os dias, horas, minutos e segundos"""
    timedelta_str = ''
    days, hours, = dt_timedelta.days, dt_timedelta.seconds // 3600
    minutes, seconds = (dt_timedelta.seconds // 60) % 60, dt_timedelta.seconds % 60
    timedelta_str += str(days) + ' dia :' if days > 0 else ''
    timedelta_str += str(hours) + ' h : ' if hours > 0 else ''
    timedelta_str += str(minutes) + ' m :' if minutes > 0 else ''
    timedelta_str += ' ' + str(seconds) + ' s ' if seconds > 0 else ''
    return timedelta_str[0:-1]



import decimal
import re
import json
import flask


def extract_json(content):
    string_json = str(content)
    # Procura se o json esta dentro de um objeto data
    # Caso esteja extrai esse json
    if re.search('data', string_json[0:7]):
        string_json = str(string_json.replace("{\"data\": \"", "", 1))
        string_json = str(string_json.replace("\"}}\"}", "\"}}", 1))
        string_json = str(string_json.replace('\\', ''))
    return json.loads(string_json)


def remove_caracteres_invalidos(str_json):
    return re.sub(u'[^{}"''_,.a-zA-Z0-9áéíóúÁÉÍÓÚâêîôÂÊÎÔãõÃÕçÇ: ]', '', str_json)


class MyJSONEncoder(flask.json.JSONEncoder):
    """"Criado para suportar decimal no json"""
    
    def default(self, obj):
        if isinstance(obj, decimal.Decimal):
            # Convert decimal instances to strings.
            return float(obj)
        return super(MyJSONEncoder, self).default(obj)
